@@ -35,23 +35,26 @@ To debug in IntelliJ Idea, open the 'Maven Projects' tool window (View
 version = "2020.2"
 
 val targets = arrayOf(
-        "linux/amd64/1.16",
-        "linux/amd64/1.17",
         "linux/amd64/1.18",
+        "linux/amd64/1.19",
+        "linux/amd64/1.20",
         "linux/amd64/tip",
 
-        "linux/386/1.18",
+        "linux/386/1.20",
 
-        "linux/arm64/1.18",
+        "linux/arm64/1.20",
         "linux/arm64/tip",
 
-        "windows/amd64/1.18",
+        "windows/amd64/1.20",
         "windows/amd64/tip",
 
-        "mac/amd64/1.18",
+        "windows/arm64/1.20",
+        "windows/arm64/tip",
+
+        "mac/amd64/1.20",
         "mac/amd64/tip",
 
-        "mac/arm64/1.18",
+        "mac/arm64/1.20",
         "mac/arm64/tip"
 )
 
