@@ -14,20 +14,22 @@ import (
 // ErrNoThreads core file did not contain any threads.
 var ErrNoThreads = errors.New("no threads found in core file")
 
-// A splicedMemory represents a memory space formed from multiple regions,
+// A SplicedMemory represents a memory space formed from multiple regions,
 // each of which may override previously regions. For example, in the following
 // core, the program text was loaded at 0x400000:
 // Start               End                 Page Offset
 // 0x0000000000400000  0x000000000044f000  0x0000000000000000
 // but then it's partially overwritten with an RW mapping whose data is stored
 // in the core file:
-// Type           Offset             VirtAddr           PhysAddr
-//                FileSiz            MemSiz              Flags  Align
-// LOAD           0x0000000000004000 0x000000000049a000 0x0000000000000000
-//                0x0000000000002000 0x0000000000002000  RW     1000
+//
+//	Type           Offset             VirtAddr           PhysAddr
+//	               FileSiz            MemSiz              Flags  Align
+//	LOAD           0x0000000000004000 0x000000000049a000 0x0000000000000000
+//	               0x0000000000002000 0x0000000000002000  RW     1000
+//
 // This can be represented in a SplicedMemory by adding the original region,
 // then putting the RW mapping on top of it.
-type splicedMemory struct {
+type SplicedMemory struct {
 	readers []readerEntry
 }
 
@@ -38,7 +40,7 @@ type readerEntry struct {
 }
 
 // Add adds a new region to the SplicedMemory, which may override existing regions.
-func (r *splicedMemory) Add(reader proc.MemoryReader, off, length uint64) {
+func (r *SplicedMemory) Add(reader proc.MemoryReader, off, length uint64) {
 	if length == 0 {
 		return
 	}
@@ -98,7 +100,7 @@ func (r *splicedMemory) Add(reader proc.MemoryReader, off, length uint64) {
 }
 
 // ReadMemory implements MemoryReader.ReadMemory.
-func (r *splicedMemory) ReadMemory(buf []byte, addr uint64) (n int, err error) {
+func (r *SplicedMemory) ReadMemory(buf []byte, addr uint64) (n int, err error) {
 	started := false
 	for _, entry := range r.readers {
 		if entry.offset+entry.length <= addr {
@@ -199,10 +201,10 @@ var openFns = []openFn{readLinuxOrPlatformIndependentCore, readAMD64Minidump}
 // any of the supported formats.
 var ErrUnrecognizedFormat = errors.New("unrecognized core format")
 
-// OpenCore will open the core file and return a Process struct.
+// OpenCore will open the core file and return a *proc.TargetGroup.
 // If the DWARF information cannot be found in the binary, Delve will look
 // for external debug files in the directories passed in.
-func OpenCore(corePath, exePath string, debugInfoDirs []string) (*proc.Target, error) {
+func OpenCore(corePath, exePath string, debugInfoDirs []string) (*proc.TargetGroup, error) {
 	var p *process
 	var currentThread proc.Thread
 	var err error
@@ -220,12 +222,13 @@ func OpenCore(corePath, exePath string, debugInfoDirs []string) (*proc.Target, e
 		return nil, ErrNoThreads
 	}
 
-	return proc.NewTarget(p, p.pid, currentThread, proc.NewTargetConfig{
-		Path:                exePath,
+	grp, addTarget := proc.NewGroup(p, proc.NewTargetGroupConfig{
 		DebugInfoDirs:       debugInfoDirs,
 		DisableAsyncPreempt: false,
-		StopReason:          proc.StopAttached,
-		CanDump:             false})
+		CanDump:             false,
+	})
+	_, err = addTarget(p, p.pid, currentThread, exePath, proc.StopAttached)
+	return grp, err
 }
 
 // BinInfo will return the binary info.
@@ -248,7 +251,9 @@ func (p *process) WriteBreakpoint(*proc.Breakpoint) error {
 func (p *process) Recorded() (bool, string) { return true, "" }
 
 // Restart will only return an error for core files, as they are not executing.
-func (p *process) Restart(string) (proc.Thread, error) { return nil, ErrContinueCore }
+func (p *process) Restart(*proc.ContinueOnceContext, string) (proc.Thread, error) {
+	return nil, ErrContinueCore
+}
 
 // ChangeDirection will only return an error as you cannot continue a core process.
 func (p *process) ChangeDirection(proc.Direction) error { return ErrContinueCore }
@@ -304,6 +309,11 @@ func (p *process) WriteMemory(addr uint64, data []byte) (int, error) {
 	return 0, ErrWriteCore
 }
 
+// FollowExec enables (or disables) follow exec mode
+func (p *process) FollowExec(bool) error {
+	return nil
+}
+
 // ProcessMemory returns the memory of this thread's process.
 func (t *thread) ProcessMemory() proc.MemoryReadWriter {
 	return t.p
@@ -355,18 +365,17 @@ func (t *thread) StepInstruction() error {
 	return ErrContinueCore
 }
 
-// Blocked will return false always for core files as there is
-// no execution.
-func (t *thread) Blocked() bool {
-	return false
-}
-
 // SetCurrentBreakpoint will always just return nil
 // for core files, as there are no breakpoints in core files.
 func (t *thread) SetCurrentBreakpoint(adjustPC bool) error {
 	return nil
 }
 
+// SoftExc returns true if this thread received a software exception during the last resume.
+func (t *thread) SoftExc() bool {
+	return false
+}
+
 // Common returns a struct containing common information
 // across thread implementations.
 func (t *thread) Common() *proc.CommonThread {
@@ -391,7 +400,7 @@ func (t *thread) SetDX(uint64) error {
 	return ErrChangeRegisterCore
 }
 
-// ChangeRegs will always return an error, you cannot
+// SetReg will always return an error, you cannot
 // change register values when debugging core files.
 func (t *thread) SetReg(regNum uint64, reg *op.DwarfRegister) error {
 	return ErrChangeRegisterCore
@@ -414,9 +423,7 @@ func (p *process) ClearInternalBreakpoints() error {
 	return nil
 }
 
-// ContinueOnce will always return an error because you
-// cannot control execution of a core file.
-func (p *process) ContinueOnce() (proc.Thread, proc.StopReason, error) {
+func (*process) ContinueOnce(cctx *proc.ContinueOnceContext) (proc.Thread, proc.StopReason, error) {
 	return nil, proc.StopUnknown, ErrContinueCore
 }
 
@@ -428,7 +435,7 @@ func (p *process) StepInstruction() error {
 
 // RequestManualStop will return nil and have no effect
 // as you cannot control execution of a core file.
-func (p *process) RequestManualStop() error {
+func (p *process) RequestManualStop(cctx *proc.ContinueOnceContext) error {
 	return nil
 }
 
