@@ -44,10 +44,6 @@ const (
 	debugCallFunctionNamePrefix2 = "runtime.debugCall"
 	maxDebugCallVersion          = 2
 	maxArgFrameSize              = 65535
-
-	// maxRegArgBytes is extra padding for ABI1 call injections, equivalent to
-	// the maximum space occupied by register arguments.
-	maxRegArgBytes = 9*8 + 15*8 // TODO: Make this generic for other platforms.
 )
 
 var (
@@ -150,11 +146,16 @@ func (callCtx *callContext) doReturn(ret *Variable, err error) {
 // EvalExpressionWithCalls is like EvalExpression but allows function calls in 'expr'.
 // Because this can only be done in the current goroutine, unlike
 // EvalExpression, EvalExpressionWithCalls is not a method of EvalScope.
-func EvalExpressionWithCalls(t *Target, g *G, expr string, retLoadCfg LoadConfig, checkEscape bool) error {
+func EvalExpressionWithCalls(grp *TargetGroup, g *G, expr string, retLoadCfg LoadConfig, checkEscape bool) error {
+	t := grp.Selected
 	bi := t.BinInfo()
 	if !t.SupportsFunctionCalls() {
 		return errFuncCallUnsupportedBackend
 	}
+	producer := bi.Producer()
+	if producer == "" || !goversion.ProducerAfterOrEqual(bi.Producer(), 1, 12) {
+		return errFuncCallUnsupported
+	}
 
 	// check that the target goroutine is running
 	if g == nil {
@@ -205,7 +206,7 @@ func EvalExpressionWithCalls(t *Target, g *G, expr string, retLoadCfg LoadConfig
 
 	contReq, ok := <-continueRequest
 	if contReq.cont {
-		return t.Continue()
+		return grp.Continue()
 	}
 
 	return finishEvalExpressionWithCalls(t, g, contReq, ok)
@@ -270,7 +271,7 @@ func evalFunctionCall(scope *EvalScope, node *ast.CallExpr) (*Variable, error) {
 	if thread == nil {
 		// We are doing a nested function call and using Go 1.15, the original
 		// injection goroutine was suspended and now we are using a different
-		// goroutine, evaluation still happend on the original goroutine but we
+		// goroutine, evaluation still happened on the original goroutine but we
 		// need to use a different thread to do the nested call injection.
 		thread = scope.callCtx.injectionThread
 		g2, err := GetG(thread)
@@ -303,10 +304,10 @@ func evalFunctionCall(scope *EvalScope, node *ast.CallExpr) (*Variable, error) {
 	if err != nil {
 		return nil, err
 	}
-	if regs.SP()-256 <= stacklo {
+	if regs.SP()-bi.Arch.debugCallMinStackSize <= stacklo {
 		return nil, errNotEnoughStack
 	}
-	protocolReg, ok := debugCallProtocolReg(dbgcallversion)
+	protocolReg, ok := debugCallProtocolReg(bi.Arch.Name, dbgcallversion)
 	if !ok {
 		return nil, errFuncCallUnsupported
 	}
@@ -324,12 +325,44 @@ func evalFunctionCall(scope *EvalScope, node *ast.CallExpr) (*Variable, error) {
 		return nil, err
 	}
 
-	if err := callOP(bi, thread, regs, dbgcallfn.Entry); err != nil {
-		return nil, err
-	}
-	// write the desired argument frame size at SP-(2*pointer_size) (the extra pointer is the saved PC)
-	if err := writePointer(bi, scope.Mem, regs.SP()-3*uint64(bi.Arch.PtrSize()), uint64(fncall.argFrameSize)); err != nil {
-		return nil, err
+	switch bi.Arch.Name {
+	case "amd64":
+		if err := callOP(bi, thread, regs, dbgcallfn.Entry); err != nil {
+			return nil, err
+		}
+		// write the desired argument frame size at SP-(2*pointer_size) (the extra pointer is the saved PC)
+		if err := writePointer(bi, scope.Mem, regs.SP()-3*uint64(bi.Arch.PtrSize()), uint64(fncall.argFrameSize)); err != nil {
+			return nil, err
+		}
+	case "arm64":
+		// debugCallV2 on arm64 needs a special call sequence, callOP can not be used
+		sp := regs.SP()
+		sp -= 2 * uint64(bi.Arch.PtrSize())
+		if err := setSP(thread, sp); err != nil {
+			return nil, err
+		}
+		if err := writePointer(bi, scope.Mem, sp, regs.LR()); err != nil {
+			return nil, err
+		}
+		if err := setLR(thread, regs.PC()); err != nil {
+			return nil, err
+		}
+		if err := writePointer(bi, scope.Mem, sp-uint64(2*bi.Arch.PtrSize()), uint64(fncall.argFrameSize)); err != nil {
+			return nil, err
+		}
+		regs, err = thread.Registers()
+		if err != nil {
+			return nil, err
+		}
+		regs, err = regs.Copy()
+		if err != nil {
+			return nil, err
+		}
+		fncall.savedRegs = regs
+		err = setPC(thread, dbgcallfn.Entry)
+		if err != nil {
+			return nil, err
+		}
 	}
 
 	fncallLog("function call initiated %v frame size %d goroutine %d (thread %d)", fncall.fn, fncall.argFrameSize, scope.g.ID, thread.ThreadID())
@@ -436,16 +469,26 @@ func writePointer(bi *BinaryInfo, mem MemoryReadWriter, addr, val uint64) error
 // * changes the value of PC to callAddr
 // Note: regs are NOT updated!
 func callOP(bi *BinaryInfo, thread Thread, regs Registers, callAddr uint64) error {
-	sp := regs.SP()
-	// push PC on the stack
-	sp -= uint64(bi.Arch.PtrSize())
-	if err := setSP(thread, sp); err != nil {
-		return err
+	switch bi.Arch.Name {
+	case "amd64":
+		sp := regs.SP()
+		// push PC on the stack
+		sp -= uint64(bi.Arch.PtrSize())
+		if err := setSP(thread, sp); err != nil {
+			return err
+		}
+		if err := writePointer(bi, thread.ProcessMemory(), sp, regs.PC()); err != nil {
+			return err
+		}
+		return setPC(thread, callAddr)
+	case "arm64":
+		if err := setLR(thread, regs.PC()); err != nil {
+			return err
+		}
+		return setPC(thread, callAddr)
+	default:
+		panic("not implemented")
 	}
-	if err := writePointer(bi, thread.ProcessMemory(), sp, regs.PC()); err != nil {
-		return err
-	}
-	return setPC(thread, callAddr)
 }
 
 // funcCallEvalFuncExpr evaluates expr.Fun and returns the function that we're trying to call.
@@ -547,6 +590,9 @@ func funcCallEvalArgs(scope *EvalScope, fncall *functionCallState, formalScope *
 
 		actualArg, err := scope.evalAST(fncall.expr.Args[i])
 		if err != nil {
+			if _, ispanic := err.(fncallPanicErr); ispanic {
+				return err
+			}
 			return fmt.Errorf("error evaluating %q as argument %s in function %s: %v", exprToString(fncall.expr.Args[i]), formalArg.name, fncall.fn.Name, err)
 		}
 		actualArg.Name = exprToString(fncall.expr.Args[i])
@@ -573,7 +619,7 @@ func funcCallCopyOneArg(scope *EvalScope, fncall *functionCallState, actualArg *
 		}
 	}
 
-	//TODO(aarzilli): autmoatic wrapping in interfaces for cases not handled
+	//TODO(aarzilli): automatic wrapping in interfaces for cases not handled
 	// by convertToEface.
 
 	var formalArgVar *Variable
@@ -599,9 +645,6 @@ func funcCallArgs(fn *Function, bi *BinaryInfo, includeRet bool) (argFrameSize i
 		return 0, nil, fmt.Errorf("DWARF read error: %v", err)
 	}
 
-	producer := bi.Producer()
-	trustArgOrder := producer != "" && goversion.ProducerAfterOrEqual(bi.Producer(), 1, 12)
-
 	if bi.regabi && fn.cu.optimized && fn.Name != "runtime.mallocgc" {
 		// Debug info for function arguments on optimized functions is currently
 		// too incomplete to attempt injecting calls to arbitrary optimized
@@ -630,7 +673,7 @@ func funcCallArgs(fn *Function, bi *BinaryInfo, includeRet bool) (argFrameSize i
 		if bi.regabi {
 			formalArg, err = funcCallArgRegABI(fn, bi, entry, argname, typ, &argFrameSize)
 		} else {
-			formalArg, err = funcCallArgOldABI(fn, bi, entry, argname, typ, trustArgOrder, &argFrameSize)
+			formalArg, err = funcCallArgOldABI(fn, bi, entry, argname, typ, &argFrameSize)
 		}
 		if err != nil {
 			return 0, nil, err
@@ -655,7 +698,7 @@ func funcCallArgs(fn *Function, bi *BinaryInfo, includeRet bool) (argFrameSize i
 		// See: https://github.com/go-delve/delve/pull/2451#discussion_r665761531
 		// TODO: Make this generic for other platforms.
 		argFrameSize = alignAddr(argFrameSize, 8)
-		argFrameSize += maxRegArgBytes
+		argFrameSize += int64(bi.Arch.maxRegArgBytes)
 	}
 
 	sort.Slice(formalArgs, func(i, j int) bool {
@@ -665,7 +708,7 @@ func funcCallArgs(fn *Function, bi *BinaryInfo, includeRet bool) (argFrameSize i
 	return argFrameSize, formalArgs, nil
 }
 
-func funcCallArgOldABI(fn *Function, bi *BinaryInfo, entry reader.Variable, argname string, typ godwarf.Type, trustArgOrder bool, pargFrameSize *int64) (*funcCallArg, error) {
+func funcCallArgOldABI(fn *Function, bi *BinaryInfo, entry reader.Variable, argname string, typ godwarf.Type, pargFrameSize *int64) (*funcCallArg, error) {
 	const CFA = 0x1000
 	var off int64
 
@@ -684,10 +727,6 @@ func funcCallArgOldABI(fn *Function, bi *BinaryInfo, entry reader.Variable, argn
 		off -= CFA
 	}
 	if err != nil {
-		if !trustArgOrder {
-			return nil, err
-		}
-
 		// With Go version 1.12 or later we can trust that the arguments appear
 		// in the same order as declared, which means we can calculate their
 		// address automatically.
@@ -721,6 +760,9 @@ func alignAddr(addr, align int64) int64 {
 }
 
 func escapeCheck(v *Variable, name string, stack stack) error {
+	if v.Unreadable != nil {
+		return fmt.Errorf("escape check for %s failed, variable unreadable: %v", name, v.Unreadable)
+	}
 	switch v.Kind {
 	case reflect.Ptr:
 		var w *Variable
@@ -804,9 +846,13 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 	}
 
 	switch regval {
-	case debugCallRegPrecheckFailed:
+	case debugCallRegPrecheckFailed: // 8
+		archoff := uint64(0)
+		if bi.Arch.Name == "arm64" {
+			archoff = 8
+		}
 		// get error from top of the stack and return it to user
-		errvar, err := readTopstackVariable(p, thread, regs, "string", loadFullValue)
+		errvar, err := readStackVariable(p, thread, regs, archoff, "string", loadFullValue)
 		if err != nil {
 			fncall.err = fmt.Errorf("could not get precheck error reason: %v", err)
 			break
@@ -814,8 +860,9 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 		errvar.Name = "err"
 		fncall.err = fmt.Errorf("%v", constant.StringVal(errvar.Value))
 
-	case debugCallRegCompleteCall:
+	case debugCallRegCompleteCall: // 0
 		p.fncallForG[callScope.g.ID].startThreadID = 0
+
 		// evaluate arguments of the target function, copy them into its argument frame and call the function
 		if fncall.fn == nil || fncall.receiver != nil || fncall.closureAddr != 0 {
 			// if we couldn't figure out which function we are calling before
@@ -846,6 +893,10 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 		}
 		cfa := regs.SP()
 		oldpc := regs.PC()
+		var oldlr uint64
+		if bi.Arch.Name == "arm64" {
+			oldlr = regs.LR()
+		}
 		callOP(bi, thread, regs, fncall.fn.Entry)
 		formalScope, err := GoroutineScope(callScope.target, thread)
 		if formalScope != nil && formalScope.Regs.CFA != int64(cfa) {
@@ -858,14 +909,22 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 
 		if err != nil {
 			// rolling back the call, note: this works because we called regs.Copy() above
-			setSP(thread, cfa)
-			setPC(thread, oldpc)
+			switch bi.Arch.Name {
+			case "amd64":
+				setSP(thread, cfa)
+				setPC(thread, oldpc)
+			case "arm64":
+				setLR(thread, oldlr)
+				setPC(thread, oldpc)
+			default:
+				panic("not implemented")
+			}
 			fncall.err = err
 			fncall.lateCallFailure = true
 			break
 		}
 
-	case debugCallRegRestoreRegisters:
+	case debugCallRegRestoreRegisters: // 16
 		// runtime requests that we restore the registers (all except pc and sp),
 		// this is also the last step of the function call protocol.
 		pc, sp := regs.PC(), regs.SP()
@@ -881,9 +940,20 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 		if err := stepInstructionOut(p, thread, debugCallName, debugCallName); err != nil {
 			fncall.err = fmt.Errorf("could not step out of %s: %v", debugCallName, err)
 		}
+		if bi.Arch.Name == "amd64" {
+			// The tail of debugCallV2 corrupts the state of RFLAGS, we must restore
+			// it one extra time after stepping out of it.
+			// See https://github.com/go-delve/delve/issues/2985 and
+			// TestCallInjectionFlagCorruption
+			rflags := bi.Arch.RegistersToDwarfRegisters(0, fncall.savedRegs).Uint64Val(regnum.AMD64_Rflags)
+			err := thread.SetReg(regnum.AMD64_Rflags, op.DwarfRegisterFromUint64(rflags))
+			if err != nil {
+				fncall.err = fmt.Errorf("could not restore RFLAGS register: %v", err)
+			}
+		}
 		return true
 
-	case debugCallRegReadReturn:
+	case debugCallRegReadReturn: // 1
 		// read return arguments from stack
 		if fncall.panicvar != nil || fncall.lateCallFailure {
 			break
@@ -922,10 +992,25 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 		if threadg, _ := GetG(thread); threadg != nil {
 			callScope.callCtx.stacks = append(callScope.callCtx.stacks, threadg.stack)
 		}
+		if bi.Arch.Name == "arm64" {
+			oldlr, err := readUintRaw(thread.ProcessMemory(), regs.SP(), int64(bi.Arch.PtrSize()))
+			if err != nil {
+				fncall.err = fmt.Errorf("could not restore LR: %v", err)
+				break
+			}
+			if err = setLR(thread, oldlr); err != nil {
+				fncall.err = fmt.Errorf("could not restore LR: %v", err)
+				break
+			}
+		}
 
-	case debugCallRegReadPanic:
+	case debugCallRegReadPanic: // 2
 		// read panic value from stack
-		fncall.panicvar, err = readTopstackVariable(p, thread, regs, "interface {}", callScope.callCtx.retLoadCfg)
+		archoff := uint64(0)
+		if bi.Arch.Name == "arm64" {
+			archoff = 8
+		}
+		fncall.panicvar, err = readStackVariable(p, thread, regs, archoff, "interface {}", callScope.callCtx.retLoadCfg)
 		if err != nil {
 			fncall.err = fmt.Errorf("could not get panic: %v", err)
 			break
@@ -941,7 +1026,7 @@ func funcCallStep(callScope *EvalScope, fncall *functionCallState, thread Thread
 	return false
 }
 
-func readTopstackVariable(t *Target, thread Thread, regs Registers, typename string, loadCfg LoadConfig) (*Variable, error) {
+func readStackVariable(t *Target, thread Thread, regs Registers, off uint64, typename string, loadCfg LoadConfig) (*Variable, error) {
 	bi := thread.BinInfo()
 	scope, err := ThreadScope(t, thread)
 	if err != nil {
@@ -951,7 +1036,7 @@ func readTopstackVariable(t *Target, thread Thread, regs Registers, typename str
 	if err != nil {
 		return nil, err
 	}
-	v := newVariable("", regs.SP(), typ, scope.BinInfo, scope.Mem)
+	v := newVariable("", regs.SP()+off, typ, scope.BinInfo, scope.Mem)
 	v.loadValue(loadCfg)
 	if v.Unreadable != nil {
 		return nil, v.Unreadable
@@ -960,7 +1045,7 @@ func readTopstackVariable(t *Target, thread Thread, regs Registers, typename str
 	return v, nil
 }
 
-// fakeEntryScope alters scope to pretend that we are at the entry point of
+// fakeFunctionEntryScope alters scope to pretend that we are at the entry point of
 // fn and CFA and SP are the ones passed as argument.
 // This function is used to create a scope for a call frame that doesn't
 // exist anymore, to read the return variables of an injected function call,
@@ -1037,7 +1122,11 @@ func isCallInjectionStop(t *Target, thread Thread, loc *Location) bool {
 		// call injection just started, did not make any progress before being interrupted by a concurrent breakpoint.
 		return false
 	}
-	text, err := disassembleCurrentInstruction(t, thread, -1)
+	off := int64(0)
+	if thread.BinInfo().Arch.breakInstrMovesPC {
+		off = -int64(len(thread.BinInfo().Arch.breakpointInstruction))
+	}
+	text, err := disassembleCurrentInstruction(t, thread, off)
 	if err != nil || len(text) <= 0 {
 		return false
 	}
@@ -1066,6 +1155,11 @@ func callInjectionProtocol(t *Target, threads []Thread) (done bool, err error) {
 			return false, err
 		}
 
+		arch := thread.BinInfo().Arch
+		if !arch.breakInstrMovesPC {
+			setPC(thread, loc.PC+uint64(len(arch.breakpointInstruction)))
+		}
+
 		fncallLog("step for injection on goroutine %d (current) thread=%d (location %s)", g.ID, thread.ThreadID(), loc.Fn.Name)
 		callinj.continueCompleted <- g
 		contReq, ok := <-callinj.continueRequest
@@ -1120,8 +1214,8 @@ func findCallInjectionStateForThread(t *Target, thread Thread) (*G, *callInjecti
 func debugCallFunction(bi *BinaryInfo) (*Function, int) {
 	for version := maxDebugCallVersion; version >= 1; version-- {
 		name := debugCallFunctionNamePrefix2 + "V" + strconv.Itoa(version)
-		fn, ok := bi.LookupFunc[name]
-		if ok && fn != nil {
+		fn := bi.lookupOneFunc(name)
+		if fn != nil {
 			return fn, version
 		}
 	}
@@ -1131,23 +1225,40 @@ func debugCallFunction(bi *BinaryInfo) (*Function, int) {
 // debugCallProtocolReg returns the register ID (as defined in pkg/dwarf/regnum)
 // of the register used in the debug call protocol, given the debug call version.
 // Also returns a bool indicating whether the version is supported.
-func debugCallProtocolReg(version int) (uint64, bool) {
-	// TODO(aarzilli): make this generic when call injection is supported on other architectures.
-	var protocolReg uint64
-	switch version {
-	case 1:
-		protocolReg = regnum.AMD64_Rax
-	case 2:
-		protocolReg = regnum.AMD64_R12
+func debugCallProtocolReg(archName string, version int) (uint64, bool) {
+	switch archName {
+	case "amd64":
+		var protocolReg uint64
+		switch version {
+		case 1:
+			protocolReg = regnum.AMD64_Rax
+		case 2:
+			protocolReg = regnum.AMD64_R12
+		default:
+			return 0, false
+		}
+		return protocolReg, true
+	case "arm64":
+		if version == 2 {
+			return regnum.ARM64_X0 + 20, true
+		}
+		return 0, false
 	default:
 		return 0, false
 	}
-	return protocolReg, true
 }
 
-type fakeEntry map[dwarf.Attr]interface{}
+type fakeEntry map[dwarf.Attr]*dwarf.Field
 
 func (e fakeEntry) Val(attr dwarf.Attr) interface{} {
+	if e[attr] == nil {
+		return nil
+	}
+
+	return e[attr].Val
+}
+
+func (e fakeEntry) AttrField(attr dwarf.Attr) *dwarf.Field {
 	return e[attr]
 }
 
@@ -1170,11 +1281,11 @@ func regabiMallocgcWorkaround(bi *BinaryInfo) ([]*godwarf.Tree, error) {
 		if err1 != nil {
 			return nil
 		}
-		var e fakeEntry = map[dwarf.Attr]interface{}{
-			dwarf.AttrName:     name,
-			dwarf.AttrType:     typ.Common().Offset,
-			dwarf.AttrLocation: []byte{byte(op.DW_OP_reg0) + byte(reg)},
-			dwarf.AttrVarParam: isret,
+		var e fakeEntry = map[dwarf.Attr]*dwarf.Field{
+			dwarf.AttrName:     &dwarf.Field{Attr: dwarf.AttrName, Val: name, Class: dwarf.ClassString},
+			dwarf.AttrType:     &dwarf.Field{Attr: dwarf.AttrType, Val: typ.Common().Offset, Class: dwarf.ClassReference},
+			dwarf.AttrLocation: &dwarf.Field{Attr: dwarf.AttrLocation, Val: []byte{byte(op.DW_OP_reg0) + byte(reg)}, Class: dwarf.ClassBlock},
+			dwarf.AttrVarParam: &dwarf.Field{Attr: dwarf.AttrVarParam, Val: isret, Class: dwarf.ClassFlag},
 		}
 
 		return &godwarf.Tree{
@@ -1183,12 +1294,25 @@ func regabiMallocgcWorkaround(bi *BinaryInfo) ([]*godwarf.Tree, error) {
 		}
 	}
 
-	r := []*godwarf.Tree{
-		m("size", t("uintptr"), regnum.AMD64_Rax, false),
-		m("typ", t("*runtime._type"), regnum.AMD64_Rbx, false),
-		m("needzero", t("bool"), regnum.AMD64_Rcx, false),
-		m("~r1", t("unsafe.Pointer"), regnum.AMD64_Rax, true),
+	switch bi.Arch.Name {
+	case "amd64":
+		r := []*godwarf.Tree{
+			m("size", t("uintptr"), regnum.AMD64_Rax, false),
+			m("typ", t("*runtime._type"), regnum.AMD64_Rbx, false),
+			m("needzero", t("bool"), regnum.AMD64_Rcx, false),
+			m("~r1", t("unsafe.Pointer"), regnum.AMD64_Rax, true),
+		}
+		return r, err1
+	case "arm64":
+		r := []*godwarf.Tree{
+			m("size", t("uintptr"), regnum.ARM64_X0, false),
+			m("typ", t("*runtime._type"), regnum.ARM64_X0+1, false),
+			m("needzero", t("bool"), regnum.ARM64_X0+2, false),
+			m("~r1", t("unsafe.Pointer"), regnum.ARM64_X0, true),
+		}
+		return r, err1
+	default:
+		// do nothing
+		return nil, nil
 	}
-
-	return r, err1
 }
