@@ -1,10 +1,17 @@
 package proc
 
 import (
+	"sync"
+
 	"github.com/go-delve/delve/pkg/elfwriter"
 	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
 )
 
+// ProcessGroup is a group of processes that are resumed at the same time.
+type ProcessGroup interface {
+	ContinueOnce(*ContinueOnceContext) (Thread, StopReason, error)
+}
+
 // Process represents the target of the debugger. This
 // target could be a system process, core file, etc.
 //
@@ -13,18 +20,9 @@ import (
 // There is one exception to this rule: it is safe to call RequestManualStop
 // concurrently with ContinueOnce.
 type Process interface {
-	// ResumeNotify specifies a channel that will be closed the next time
-	// ContinueOnce finishes resuming the target.
-	ResumeNotify(chan<- struct{})
 	BinInfo() *BinaryInfo
 	EntryPoint() (uint64, error)
 
-	// RequestManualStop attempts to stop all the process' threads.
-	RequestManualStop() error
-	// CheckAndClearManualStopRequest returns true the first time it's called
-	// after a call to RequestManualStop.
-	CheckAndClearManualStopRequest() bool
-
 	FindThread(threadID int) (Thread, bool)
 	ThreadList() []Thread
 
@@ -45,7 +43,9 @@ type ProcessInternal interface {
 	// ErrProcessExited or ErrProcessDetached).
 	Valid() (bool, error)
 	Detach(bool) error
-	ContinueOnce() (trapthread Thread, stopReason StopReason, err error)
+
+	// RequestManualStop attempts to stop all the process' threads.
+	RequestManualStop(cctx *ContinueOnceContext) error
 
 	WriteBreakpoint(*Breakpoint) error
 	EraseBreakpoint(*Breakpoint) error
@@ -62,6 +62,9 @@ type ProcessInternal interface {
 
 	// StartCallInjection notifies the backend that we are about to inject a function call.
 	StartCallInjection() (func(), error)
+
+	// FollowExec enables (or disables) follow exec mode
+	FollowExec(bool) error
 }
 
 // RecordingManipulation is an interface for manipulating process recordings.
@@ -69,7 +72,7 @@ type RecordingManipulation interface {
 	// Recorded returns true if the current process is a recording and the path
 	// to the trace directory.
 	Recorded() (recorded bool, tracedir string)
-	// Direction changes execution direction.
+	// ChangeDirection changes execution direction.
 	ChangeDirection(Direction) error
 	// GetDirection returns the current direction of execution.
 	GetDirection() Direction
@@ -93,7 +96,7 @@ type RecordingManipulationInternal interface {
 	// If pos starts with 'c' it's a checkpoint ID, otherwise it's an event
 	// number.
 	// Returns the new current thread after the restart has completed.
-	Restart(pos string) (Thread, error)
+	Restart(cctx *ContinueOnceContext, pos string) (Thread, error)
 }
 
 // Direction is the direction of execution for the target process.
@@ -112,3 +115,30 @@ type Checkpoint struct {
 	When  string
 	Where string
 }
+
+// ContinueOnceContext is an object passed to ContinueOnce that the backend
+// can use to communicate with the target layer.
+type ContinueOnceContext struct {
+	ResumeChan chan<- struct{}
+	StopMu     sync.Mutex
+	// manualStopRequested is set if all the threads in the process were
+	// signalled to stop as a result of a Halt API call. Used to disambiguate
+	// why a thread is found to have stopped.
+	manualStopRequested bool
+}
+
+// CheckAndClearManualStopRequest will check for a manual
+// stop and then clear that state.
+func (cctx *ContinueOnceContext) CheckAndClearManualStopRequest() bool {
+	cctx.StopMu.Lock()
+	defer cctx.StopMu.Unlock()
+	msr := cctx.manualStopRequested
+	cctx.manualStopRequested = false
+	return msr
+}
+
+func (cctx *ContinueOnceContext) GetManualStopRequested() bool {
+	cctx.StopMu.Lock()
+	defer cctx.StopMu.Unlock()
+	return cctx.manualStopRequested
+}
