@@ -3,7 +3,6 @@ package native
 import (
 	"fmt"
 	"os"
-	"path/filepath"
 	"syscall"
 	"unsafe"
 
@@ -11,7 +10,6 @@ import (
 
 	"github.com/go-delve/delve/pkg/proc"
 	"github.com/go-delve/delve/pkg/proc/internal/ebpf"
-	"github.com/go-delve/delve/pkg/proc/winutil"
 )
 
 // osProcessDetails holds Windows specific information.
@@ -25,11 +23,8 @@ type osProcessDetails struct {
 func (os *osProcessDetails) Close() {}
 
 // Launch creates and begins debugging a new process.
-func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ string, redirects [3]string) (*proc.Target, error) {
-	argv0Go, err := filepath.Abs(cmd[0])
-	if err != nil {
-		return nil, err
-	}
+func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ string, redirects [3]string) (*proc.TargetGroup, error) {
+	argv0Go := cmd[0]
 
 	env := proc.DisableAsyncPreemptEnv()
 
@@ -38,6 +33,11 @@ func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ strin
 		return nil, err
 	}
 
+	creationFlags := uint32(_DEBUG_ONLY_THIS_PROCESS)
+	if flags&proc.LaunchForeground == 0 {
+		creationFlags |= syscall.CREATE_NEW_PROCESS_GROUP
+	}
+
 	var p *os.Process
 	dbp := newProcess(0)
 	dbp.execPtraceFunc(func() {
@@ -45,7 +45,7 @@ func Launch(cmd []string, wd string, flags proc.LaunchFlags, _ []string, _ strin
 			Dir:   wd,
 			Files: []*os.File{stdin, stdout, stderr},
 			Sys: &syscall.SysProcAttr{
-				CreationFlags: _DEBUG_ONLY_THIS_PROCESS,
+				CreationFlags: creationFlags,
 			},
 			Env: env,
 		}
@@ -137,8 +137,20 @@ func findExePath(pid int) (string, error) {
 	}
 }
 
+var debugPrivilegeRequested = false
+
 // Attach to an existing process with the given PID.
-func Attach(pid int, _ []string) (*proc.Target, error) {
+func Attach(pid int, _ []string) (*proc.TargetGroup, error) {
+	var aperr error
+	if !debugPrivilegeRequested {
+		debugPrivilegeRequested = true
+		// The following call will only work if the user is an administrator
+		// has the "Debug Programs" privilege in Local security settings.
+		// Since this privilege is not needed to debug processes owned by the
+		// current user, do not complain about this unless attach actually fails.
+		aperr = acquireDebugPrivilege()
+	}
+
 	dbp := newProcess(pid)
 	var err error
 	dbp.execPtraceFunc(func() {
@@ -146,6 +158,9 @@ func Attach(pid int, _ []string) (*proc.Target, error) {
 		err = _DebugActiveProcess(uint32(pid))
 	})
 	if err != nil {
+		if aperr != nil {
+			return nil, fmt.Errorf("%v also %v", err, aperr)
+		}
 		return nil, err
 	}
 	exepath, err := findExePath(pid)
@@ -160,6 +175,40 @@ func Attach(pid int, _ []string) (*proc.Target, error) {
 	return tgt, nil
 }
 
+// acquireDebugPrivilege acquires the debug privilege which is needed to
+// debug other user's processes.
+// See:
+//
+//   - https://learn.microsoft.com/en-us/windows-hardware/drivers/debugger/debug-privilege
+//   - https://github.com/go-delve/delve/issues/3136
+func acquireDebugPrivilege() error {
+	var token sys.Token
+	err := sys.OpenProcessToken(sys.CurrentProcess(), sys.TOKEN_QUERY|sys.TOKEN_ADJUST_PRIVILEGES, &token)
+	if err != nil {
+		return fmt.Errorf("could not acquire debug privilege (OpenCurrentProcessToken): %v", err)
+	}
+	defer token.Close()
+
+	privName, _ := sys.UTF16FromString("SeDebugPrivilege")
+	var luid sys.LUID
+	err = sys.LookupPrivilegeValue(nil, &privName[0], &luid)
+	if err != nil {
+		return fmt.Errorf("could not acquire debug privilege  (LookupPrivilegeValue): %v", err)
+	}
+
+	var tp sys.Tokenprivileges
+	tp.PrivilegeCount = 1
+	tp.Privileges[0].Luid = luid
+	tp.Privileges[0].Attributes = sys.SE_PRIVILEGE_ENABLED
+
+	err = sys.AdjustTokenPrivileges(token, false, &tp, 0, nil, nil)
+	if err != nil {
+		return fmt.Errorf("could not acquire debug privilege (AdjustTokenPrivileges): %v", err)
+	}
+
+	return nil
+}
+
 // kill kills the process.
 func (dbp *nativeProcess) kill() error {
 	if dbp.exited {
@@ -341,6 +390,9 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 
 				if atbp {
 					dbp.os.breakThread = tid
+					if th := dbp.threads[tid]; th != nil {
+						th.os.setbp = true
+					}
 					return tid, 0, nil
 				} else {
 					continueStatus = _DBG_CONTINUE
@@ -375,7 +427,8 @@ func (dbp *nativeProcess) waitForDebugEvent(flags waitForDebugEventFlags) (threa
 	}
 }
 
-func (dbp *nativeProcess) trapWait(pid int) (*nativeThread, error) {
+func trapWait(procgrp *processGroup, pid int) (*nativeThread, error) {
+	dbp := procgrp.procs[0]
 	var err error
 	var tid, exitCode int
 	dbp.execPtraceFunc(func() {
@@ -422,12 +475,17 @@ func (dbp *nativeProcess) resume() error {
 }
 
 // stop stops all running threads threads and sets breakpoints
-func (dbp *nativeProcess) stop(trapthread *nativeThread) (*nativeThread, error) {
+func (procgrp *processGroup) stop(cctx *proc.ContinueOnceContext, trapthread *nativeThread) (*nativeThread, error) {
+	dbp := procgrp.procs[0]
 	if dbp.exited {
 		return nil, proc.ErrProcessExited{Pid: dbp.pid}
 	}
 
 	dbp.os.running = false
+	for _, th := range dbp.threads {
+		th.os.setbp = false
+	}
+	trapthread.os.setbp = true
 
 	// While the debug event that stopped the target was being propagated
 	// other target threads could generate other debug events.
@@ -444,7 +502,7 @@ func (dbp *nativeProcess) stop(trapthread *nativeThread) (*nativeThread, error)
 		return nil, err
 	}
 
-	context := winutil.NewCONTEXT()
+	context := newContext()
 
 	for _, thread := range dbp.threads {
 		thread.os.delayErr = nil
@@ -454,7 +512,7 @@ func (dbp *nativeProcess) stop(trapthread *nativeThread) (*nativeThread, error)
 			_, thread.os.delayErr = _SuspendThread(thread.os.hThread)
 			if thread.os.delayErr == nil {
 				// This call will block until the thread has stopped.
-				_ = _GetThreadContext(thread.os.hThread, context)
+				_ = thread.getContext(context)
 			}
 		}
 	}
