@@ -108,30 +108,10 @@ func New(client service.Client, conf *config.Config) *Term {
 		t.stdout.pw = &pagingWriter{w: getColorableWriter()}
 		t.stdout.colorEscapes = make(map[colorize.Style]string)
 		t.stdout.colorEscapes[colorize.NormalStyle] = terminalResetEscapeCode
-		wd := func(s string, defaultCode int) string {
-			if s == "" {
-				return fmt.Sprintf(terminalHighlightEscapeCode, defaultCode)
-			}
-			return s
-		}
-		t.stdout.colorEscapes[colorize.KeywordStyle] = conf.SourceListKeywordColor
-		t.stdout.colorEscapes[colorize.StringStyle] = wd(conf.SourceListStringColor, ansiGreen)
-		t.stdout.colorEscapes[colorize.NumberStyle] = conf.SourceListNumberColor
-		t.stdout.colorEscapes[colorize.CommentStyle] = wd(conf.SourceListCommentColor, ansiBrMagenta)
-		t.stdout.colorEscapes[colorize.ArrowStyle] = wd(conf.SourceListArrowColor, ansiYellow)
-		switch x := conf.SourceListLineColor.(type) {
-		case string:
-			t.stdout.colorEscapes[colorize.LineNoStyle] = x
-		case int:
-			if (x > ansiWhite && x < ansiBrBlack) || x < ansiBlack || x > ansiBrWhite {
-				x = ansiBlue
-			}
-			t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, x)
-		case nil:
-			t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, ansiBlue)
-		}
 	}
 
+	t.updateConfig()
+
 	if client != nil {
 		lcfg := t.loadConfig()
 		client.SetReturnValuesLoadConfig(&lcfg)
@@ -141,6 +121,47 @@ func New(client service.Client, conf *config.Config) *Term {
 	return t
 }
 
+func (t *Term) updateConfig() {
+	// These are always called together.
+	t.updateColorScheme()
+	t.updateTab()
+}
+
+func (t *Term) updateColorScheme() {
+	if t.stdout.colorEscapes == nil {
+		return
+	}
+
+	conf := t.conf
+	wd := func(s string, defaultCode int) string {
+		if s == "" {
+			return fmt.Sprintf(terminalHighlightEscapeCode, defaultCode)
+		}
+		return s
+	}
+	t.stdout.colorEscapes[colorize.KeywordStyle] = conf.SourceListKeywordColor
+	t.stdout.colorEscapes[colorize.StringStyle] = wd(conf.SourceListStringColor, ansiGreen)
+	t.stdout.colorEscapes[colorize.NumberStyle] = conf.SourceListNumberColor
+	t.stdout.colorEscapes[colorize.CommentStyle] = wd(conf.SourceListCommentColor, ansiBrMagenta)
+	t.stdout.colorEscapes[colorize.ArrowStyle] = wd(conf.SourceListArrowColor, ansiYellow)
+	t.stdout.colorEscapes[colorize.TabStyle] = wd(conf.SourceListTabColor, ansiBrBlack)
+	switch x := conf.SourceListLineColor.(type) {
+	case string:
+		t.stdout.colorEscapes[colorize.LineNoStyle] = x
+	case int:
+		if (x > ansiWhite && x < ansiBrBlack) || x < ansiBlack || x > ansiBrWhite {
+			x = ansiBlue
+		}
+		t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, x)
+	case nil:
+		t.stdout.colorEscapes[colorize.LineNoStyle] = fmt.Sprintf(terminalHighlightEscapeCode, ansiBlue)
+	}
+}
+
+func (t *Term) updateTab() {
+	t.stdout.altTabString = t.conf.Tab
+}
+
 func (t *Term) SetTraceNonInteractive() {
 	t.traceNonInteractive = true
 }
@@ -411,13 +432,16 @@ func (t *Term) promptForInput() (string, error) {
 	return l, nil
 }
 
-func yesno(line *liner.State, question string) (bool, error) {
+func yesno(line *liner.State, question, defaultAnswer string) (bool, error) {
 	for {
 		answer, err := line.Prompt(question)
 		if err != nil {
 			return false, err
 		}
 		answer = strings.ToLower(strings.TrimSpace(answer))
+		if answer == "" {
+			answer = defaultAnswer
+		}
 		switch answer {
 		case "n", "no":
 			return false, nil
@@ -448,7 +472,7 @@ func (t *Term) handleExit() (int, error) {
 	if err != nil {
 		if isErrProcessExited(err) {
 			if t.client.IsMulticlient() {
-				answer, err := yesno(t.line, "Remote process has exited. Would you like to kill the headless instance? [Y/n] ")
+				answer, err := yesno(t.line, "Remote process has exited. Would you like to kill the headless instance? [Y/n] ", "yes")
 				if err != nil {
 					return 2, io.EOF
 				}
@@ -474,7 +498,7 @@ func (t *Term) handleExit() (int, error) {
 
 		doDetach := true
 		if t.client.IsMulticlient() {
-			answer, err := yesno(t.line, "Would you like to kill the headless instance? [Y/n] ")
+			answer, err := yesno(t.line, "Would you like to kill the headless instance? [Y/n] ", "yes")
 			if err != nil {
 				return 2, io.EOF
 			}
@@ -484,7 +508,7 @@ func (t *Term) handleExit() (int, error) {
 		if doDetach {
 			kill := true
 			if t.client.AttachedToExistingProcess() {
-				answer, err := yesno(t.line, "Would you like to kill the process? [Y/n] ")
+				answer, err := yesno(t.line, "Would you like to kill the process? [Y/n] ", "yes")
 				if err != nil {
 					return 2, io.EOF
 				}
