@@ -5,11 +5,15 @@ import (
 	"io/ioutil"
 	"os"
 	"syscall"
-
-	sys "golang.org/x/sys/unix"
 )
 
-func attachErrorMessage(pid int, err error) error {
+func init() {
+	attachErrorMessage = attachErrorMessageLinux
+}
+
+//lint:file-ignore ST1005 errors here can be capitalized
+
+func attachErrorMessageLinux(pid int, err error) error {
 	fallbackerr := fmt.Errorf("could not attach to pid %d: %s", pid, err)
 	if serr, ok := err.(syscall.Errno); ok {
 		switch serr {
@@ -30,7 +34,3 @@ func attachErrorMessage(pid int, err error) error {
 	}
 	return fallbackerr
 }
-
-func stopProcess(pid int) error {
-	return sys.Kill(pid, sys.SIGSTOP)
-}
